
module Scope (	clk,					// 1 bit in
					trigger_en,		// 1 bit in
					trigger_level,		// 8 bit in
					trigger_mode,		// 4 bit in
					dig_mode,			// 1 bit in
					dig_mask,			// 8 bit in
					hyst_level,			// 4 bit in
					sample_len, 		// 13 bit in
					sample_ct_en,		//	1 bit in
					write_en,			// 1 bit in
					write_gate2,		// 1 bit in
					time_div,			// 12 bit in
					adc_data,			// 8 bit in
					ext_trigger,		// 1 bit in
					adc_seq,				// 1 bit in
					scope_stop,			// 1 bit in
					
					trigger_addr,		// 15 bit out
					scope_status,		// 2 bit out
					ram_addr,			// 12 bit out
					scope_write_addr,	// 14 bit out
					ram_data,			// 32 bit out
					ram_clk,				// 1 bit out
					ready					// 1 bit out
					);

input 			clk;
input				trigger_en;		// 1 clk impulzus Trigger engedelyezes
input  [7:0]	trigger_level; // Trigger szint
input	 [3:0]	trigger_mode;	// bit0 	-> 0 Lefuto vagy L, 	1 felfuto vagy H
										// bit1	->	0 el trigger, 		1 szint trigger
										// bit2	-> 0 int trigger, 	1 ext trigger
										// bit3	-> 0 normal, 			1 peak detect 

input				dig_mode;		// Ha 1, akkor logikai analizator modba kapcsol a trigger, ami
										// azt jelenti, hogy egyenlosegre komparal es egy 8 bit-es maszkot
										// is hasznal. Ha a maszk bit 1, akkor figyelembe veszi a komparalasnal,
										// ha 0 akkor pedig nem.
										
input	 [7:0]	dig_mask;		// Logikai analizator modban a maszk biteket tartalmazza.
										// Ha 1, akkor figyelembe veszi a trigger komparator az A/D kivalasztott bitjet, ha 0 akkor nem.
 
input	 [3:0]	hyst_level;		// Trigger hiszterezis
input [15:0]	sample_len;		// A digitalizalt minta hossza
input				sample_ct_en;	// 1 clk impulzus Az eltarolt mintak szamlalasanak engedelzezese
input				write_en;		// 1 clk impulzus A mintatarolas engedelyezese
input				write_gate2;	// 1 vagy 4 byte-os irasi mod kivaszatasa 0=4 byte, 1=1byte
input [23:0]	time_div;		// Idoalap erteke
input  [7:0]	adc_data;		// A/D konverter adatbusz
input				ext_trigger;	// Kulso trigger bemenet
input				adc_seq;			// A/D konverter bitsorrend valasztas
input				scope_stop;		// Mindenfele mukodest megallit
 
output [14:0]	trigger_addr; 		// Trigger pont cime a ram-ban. A tenyleges cim a kesleltetett
reg	 [14:0]	trigger_addr = 0;	// RAM iras miatt fugg az idoalap osztasatol. 
											// Ha a time_div = 0 akkor a trigger_addr = trigger_addr - 2
											// Ha a time_div = 1 akkor a trigger_addr = trigger_addr - 1
											// Ha a time_div > 1 akkor a trigger_addr = trigger_addr 
 
output  [2:0]	scope_status;		// Mukodesi allapot jelzes
											// bit0	<- start
											// bit1  <- sample_gate
											// bit2  <- write_gate
												
output  [12:0]	ram_addr;		// A RAM cim. A DSO felol 32 bit-es szervezesu
reg	  [12:0]	ram_addr = 0;
 
output  [14:0]	scope_write_addr;		 // A RAM aktualis cime ahova az eppen eltarolt minta kerul.
reg	  [14:0]	scope_write_addr = 0; // ROLL modhoz a CPU fele 8 bit szervezesu RAM olvasashoz
 
output  [31:0]	ram_data;		// DSO-bol a RAM fele meno adat 
reg	  [31:0]	ram_data = 0;
output			ram_clk;			// RAM orajel
reg				ram_clk	= 0;
output			ready;			// A write_gate lefuto ele 1 clk ideig 1-be irja. A CPU fele IRQ lessz belole
reg				ready 	= 0;
 
// Ezekben gyulik a 32 bit-es minta mielott a RAM-ba kerulne
reg	 [7:0]	reg0	= 0;	
reg	 [7:0]	reg1	= 0;
reg 	 [7:0]	reg2	= 0;
 
reg				start		 			= 0; // Ha 1, akkor elesitve van a trigger. Amikor trigger esemeny
												  // tortenik, akkor torlodik
reg 	[15:0]	sample_ct 			= 0; // Az eltarolt mitakat szamolja
reg	 [1:0]	byte_ct	 			= 0; // A 32 bites adat aktualis byte-jat mutatja
reg	[23:0]	time_ct   			= 0; // Az idoalap szamlaloja
reg	 [1:0]	trigger_en_edge	= 0; // Trigger engedelyezes eldetektalashoz
reg	 [1:0]	write_en_edge		= 0; // A RAM-ba iras engedelyezes eldetektalasahoz
 
reg				hyst_pos				= 0; // 1 ha a trigger komparatos bemenete nagyobb mint a hiszterezis also pontja
reg				hyst_neg				= 0; // 1 ha a trigger komparatos bemenete kissebb mint a hiszterezis felso pontja

reg	 [1:0]	sample_ct_en_edge	= 0; // Mintaveteli hatar engedelyezes eldetektalashoz
reg				sample_gate 		= 0; // 1 ha engedelyezve van a mintaveteli hatar
reg				comp_out				= 0; // A trigger komparator kimenete
reg	 [1:0]	comparator_edge	= 0; // A trigger komparator kimenetenek eldetektalasahoz
reg				write_gate 			= 0; // Ha 1, akkor engedi a mintak RAM-ba irasat
reg	 [1:0]	write_gate_edge	= 0; // Az RAM iras engedelyezes eldetektalashoz
reg				ram_clk0				= 0; // RAM orajel ami majd 1 clk kesleltetessel kerul a ram-hoz
reg				time_ok				= 0; // 1 clk impulzus ha a idoalap szamlaloja elerte a time_div erteket
reg	 [7:0]	peak_lo				= 0; // legkissebb minta a PEAK-DETECT modban
reg	 [7:0]	peak_hi				= 0; // legnagzobb minta a PEAK-DETECT modban
reg				trigger_set			= 0; // trigger cim beallitasat engedelyezi
 
wire	 [1:0]	trigger_edge;		// A trigger H, L vagy el beallitas a komparatorhoz
wire				comparator_ok;		// 1 ha a trigger esemeny van
 
 
wire [7:0]	ADC;
 
// Az A/D konverter bitsorrendjenek felcserelese az adc_seq erteketol fuggoen
assign ADC[0] = ( adc_seq == 0 ) ? adc_data[7] : adc_data[0];
assign ADC[1] = ( adc_seq == 0 ) ? adc_data[6] : adc_data[1];
assign ADC[2] = ( adc_seq == 0 ) ? adc_data[5] : adc_data[2];
assign ADC[3] = ( adc_seq == 0 ) ? adc_data[4] : adc_data[3];
assign ADC[4] = ( adc_seq == 0 ) ? adc_data[3] : adc_data[4];
assign ADC[5] = ( adc_seq == 0 ) ? adc_data[2] : adc_data[5];
assign ADC[6] = ( adc_seq == 0 ) ? adc_data[1] : adc_data[6];
assign ADC[7] = ( adc_seq == 0 ) ? adc_data[0] : adc_data[7];
 
// Statusz regiszterbe a CPU fele torteno visszaolvasasra
assign scope_status[0] = start;
assign scope_status[1] = sample_gate;
assign scope_status[2] = write_gate;

// Beallitja a komparalasi modot, el vagy szint -re.
assign trigger_edge  = ( trigger_mode[1] == 0 ) ? 2'b01 : { trigger_mode[0], trigger_mode[0] }; 
// Az elozoleg beallitott komparalasi mod  eredmenye 1 ha megfelelt a feltetelnek
assign comparator_ok = ( comparator_edge[1:0] == trigger_edge[1:0] ) ? 2'b01 : 2'b00;
 
always @ ( posedge clk ) begin 

	// a ready es a ram_clk jel csak 1 clk ideig kell
	//ready 	<= 0;
	ram_clk 	<= ram_clk0;
	ram_clk0 <= 0;
	
	// DSO ready eloallitasa
	write_gate_edge[0] <= write_gate;
	write_gate_edge[1] <= write_gate_edge[0];
	//if ( write_gate_edge[1:0] == 2'b10 ) ready <= 1;
	if ( write_gate_edge[1] != write_gate_edge[0] ) ready <= ~ready;
		
	// RAM-ba iras engedelyezes eldetektalas
	write_en_edge[0] <= write_en;
	write_en_edge[1] <= write_en_edge[0]; 
	//if ( write_en_edge[1:0] == 2'b01 ) write_gate <= 1;
	if ( write_en_edge[1] != write_en_edge[0] ) write_gate <= 1;
 
	// Trigger engedelyezes eldetektalasa
	trigger_en_edge[0] <= trigger_en;
	trigger_en_edge[1] <= trigger_en_edge[0];
	//if ( trigger_en_edge[1:0] == 2'b01 ) start <= 1;
	if ( trigger_en_edge[1] != trigger_en_edge[0] ) start <= 1;
	
	// Mintaveteli hatar engedelyezes eldetektalas
	sample_ct_en_edge[0] <= sample_ct_en;
	sample_ct_en_edge[1] <= sample_ct_en_edge[0];
	//if ( sample_ct_en_edge[1:0] == 2'b01 ) begin 
	if ( sample_ct_en_edge[1] != sample_ct_en_edge[0] ) begin 
		sample_gate <= 1;
		sample_ct 	<= 0;
		trigger_addr[14:2] <= ram_addr;	// Triggerpont RAM cime
		if ( trigger_mode[3] == 0 ) begin 
			trigger_addr[1:0] <= byte_ct; // normal mode
		end 
		else begin 
			trigger_addr[1:0] <= { byte_ct[0], 1'b0 }; // peak detect mode
		end
	end 
 
	
		if ( trigger_mode[3] == 0 ) begin 
			scope_write_addr <= { ram_addr, byte_ct }; // normal mode
		end 
		else begin 
			scope_write_addr <= { ram_addr, byte_ct[0], 1'b0 }; // peak detect mode
		end
	


	// Hysteresis hatarok beallitasa
	if ( ADC <= trigger_level - hyst_level ) hyst_pos <= 1;
	if ( ADC >= trigger_level + hyst_level ) hyst_neg <= 1;
	
	// Trigger komparator
	if ( trigger_mode[2] == 0 ) begin // int trigger
		if ( dig_mode == 0 ) begin // Scope mod
			if ( ADC >= trigger_level ) begin 
				if ( hyst_pos == 1 ) begin
					comp_out <= 1; 
					hyst_pos	<= 0;
				end
			end
			else begin 
				if ( hyst_neg == 1 ) begin
					comp_out <= 0; 
					hyst_neg	<= 0;
				end
			end
		end
		else begin // Logikai analizator mod
			if ( ( ADC & dig_mask ) == ( trigger_level & dig_mask ) ) comp_out <= 1;  else	comp_out <= 0; 
		end
	end
	else begin // ext trigger
		comp_out <= ext_trigger;
	end
	
	// Trigger komparator kimenete egy 2bit-es shift regiszterbe
	// hogy az eldetektalast erzekelje
	if ( trigger_mode[0] == 0 ) comparator_edge[0] <= ~comp_out; else comparator_edge[0] <= comp_out;
	comparator_edge[1] <= comparator_edge[0];
	
	// peak detect funkciohoz a csucsertekek eltarolasa
	if	( ADC < peak_lo ) peak_lo <= ADC;
	if ( ADC > peak_hi ) peak_hi <= ADC;
	
	// Ha az idoalap idozito elerte a time_div erteket, akkor megkezdi a mintavetelezest es ha kell
	// akkor az eltarolast is.
	if ( time_ok == 1 ) begin
		
		if ( trigger_set == 1 ) begin
			trigger_set <= 0;
			trigger_addr[14:2] <= ram_addr + 1;	 // Triggerpont RAM cime
			if ( trigger_mode[3] == 0 ) begin 
				trigger_addr[1:0] <= byte_ct; // normal mode
			end 
			else begin 
				trigger_addr[1:0] <= { byte_ct[0], 1'b0 }; // peak detect mode
			end
		end
	
		// peak detect modhoz ilyenkor az aktualis mintat mindket regiszterbe be kell tolteni
		peak_lo <= ADC;
		peak_hi <= ADC;
		 if ( write_gate == 1 ) byte_ct <= byte_ct + 1; // byte szamlalo leptetese
		
		// Ha minden byte kiirasa engedelyezett, akkor attol fuggetlenul hogy
		// ervenyes-e mind a 4 regiszter kiirja a RAM-ba. 
		// Termeszetesen a normal vagy a peak-detect modnak megfelelo formaban.
		if ( write_gate == 1 && write_gate2 == 1 ) begin
			ram_clk0 <= 1;
			if ( trigger_mode[3] == 0 ) begin
				ram_data <= { ADC, reg2, reg1, reg0 };
			end
			else begin
				ram_data <= { peak_hi, peak_lo, reg1, reg0 };
			end
		end 
		
		if ( sample_gate == 1 )  sample_ct <= sample_ct + 1; // A mintaveteli hatar szamlalojanak leptetese
				
		if ( trigger_mode[3] == 0 ) begin 	// normal mode
			if ( byte_ct == 0 ) reg0 <= ADC; // 1. byte
			if ( byte_ct == 1 ) reg1 <= ADC; // 2. byte
			if ( byte_ct == 2 ) reg2 <= ADC; // 3. byte
			if ( byte_ct == 3 ) begin 			// 4. byte
				ram_data <= { ADC, reg2, reg1, reg0 }; // Ha megvan a 4 byte, akkor mehet a RAM adatregiszterbe

				if ( write_gate == 1 ) ram_clk0 <= 1; 	// Ha engedelyezve van a tarolas, akkor kiirja a RAM-ba
				if ( ram_addr == 5119  && write_gate == 1  ) ram_addr <= 0; else ram_addr <= ram_addr + 1;
				
				if ( sample_gate == 1 ) begin 
					// Figyeli a mintaveteli hatart es ha elerte, akkor leallitja a tarolast
					if ( sample_ct >= sample_len ) begin
						write_gate 	<= 0;
						sample_gate <= 0;
					end
				end
			end
		end
		else begin // peak detect mode
			if ( byte_ct[0] == 0 ) begin // 1. es a 2. byte
				reg0 <= peak_lo; 
				reg1 <= peak_hi; 
			end
			else begin // 3. es a 4. byte
				ram_data <= { peak_hi, peak_lo, reg1, reg0 };
				
				if ( write_gate == 1 ) ram_clk0 <= 1; 	// Ha engedelyezve van a tarolas, akkor kiirja a RAM-ba
				if ( ram_addr == 5119  && write_gate == 1  ) ram_addr <= 0; else ram_addr <= ram_addr + 1;
				 
				
				if ( sample_gate == 1 ) begin  // Mintaveteli hatar ellenorzese
					// Figyeli a mintaveteli hatart es ha elerte, akkor leallitja a tarolast
					if ( sample_ct >= sample_len ) begin
						write_gate 	<= 0;
						sample_gate <= 0;
					end
				end
			end
		end
	end
	
	// Idoalap leptetese
	time_ok <= 0;
	if ( time_ct == time_div ) begin
		time_ct <= 0;
		time_ok <= 1;
	end 
	else begin 
		time_ct <= time_ct + 1;	// Idoalap leptetese
	end

	// Megallita a mukodest
	if ( scope_stop == 1 ) begin
		start			<= 0;	
		write_gate	<= 0;
		sample_gate	<= 0;
	end
	
	// A trigger feltetel ok ?
	if ( comparator_ok == 1 && start == 1 )  begin
		start					 <= 0;	// start jel torles
		trigger_set			 <= 1;	// trigger cim beallitas engedelzezes
		time_ct 				 <= 0;	// Idoalap szamlalot nullazza
		sample_ct			 <= 0;	// mintaszamlalot nullazza
		time_ok				 <= 1;	// idoalap jelzes 1-be
		write_gate			 <= 1;	// RAM-ba eltarolast engedelyezi
		sample_gate			 <= 1;	// Mintaveteli hatar figyelest engedi
	end

	

end



endmodule



module quad(clk, quadA, quadB, count, set, set_data, enc_mode, enc_dir, enc_type ); 
 input 			clk; 
 input 			quadA; 
 input 			quadB; 
 input 			set;
 input 	[7:0]	set_data;
 input			enc_mode;
 input			enc_dir;
 input			enc_type;
 output 	[7:0] count;
 reg 		[7:0] count = 0; 
  
 reg 		[2:0]	quadA_delayed; 
 reg 		[2:0]	quadB_delayed; 
 
 reg		[2:0] count_2 = 2;

  
 wire count_enable 	 = quadA_delayed[1] ^ quadA_delayed[2] ^ quadB_delayed[1] ^ quadB_delayed[2]; 
 wire count_direction = quadA_delayed[1] ^ quadB_delayed[2]; 
  
 always @(posedge clk) begin 
   quadA_delayed <= { quadA_delayed[1:0], quadA }; 
   quadB_delayed <= { quadB_delayed[1:0], quadB }; 
 end 
  
 always @ ( posedge clk ) begin 
	if ( set == 0 ) begin
		if( count_enable == 1 ) begin 
			if( count_direction == 1 ) begin
				count_2 <= count_2 + 1;
				if ( count_2 >= 3 || enc_type == 0 ) begin 
					count_2 <= 2;
					if ( enc_dir == 0 ) begin 
						if ( enc_mode == 0 || ( enc_mode == 1 && ( count < 255 ) ) ) count <= count + 1; 
					end 
					else begin 
						if ( enc_mode == 0 || ( enc_mode == 1 && ( count > 0   ) ) ) count <= count - 1; 
					end
				end
			end 
			else begin
				count_2 <= count_2 - 1;
				if ( count_2 == 1 || enc_type == 0 ) begin 
					count_2 <= 2;
					if ( enc_dir == 0 ) begin 
						if ( enc_mode == 0 || ( enc_mode == 1 && ( count > 0   ) ) ) count <= count - 1; 
					end
					else begin
						if ( enc_mode == 0 || ( enc_mode == 1 && ( count < 255 ) ) ) count <= count + 1; 
					end
				end
			end
		end
	end
	else begin
		count <= set_data;
	end

 end 
endmodule

module pwm_8(clk, pwm_in, pwm_out, pwm_count_en );
 input 			clk;
 input [7:0]	pwm_in;
 input			pwm_count_en;
 output 			pwm_out;
 reg	 			pwm_out;

 reg 	 [7:0] 	pwm_ramp = 0; 

 always @(posedge clk)  begin
	if ( pwm_count_en == 1 ) pwm_ramp <= pwm_ramp + 1; 
   pwm_out 	<= ( pwm_in > pwm_ramp );
 end

endmodule

module pwm_12(clk, pwm_in, pwm_out, pwm_count_en );
 input 			clk;
 input [11:0]	pwm_in;
 input			pwm_count_en;
 output 			pwm_out;
 reg	 			pwm_out;

 reg 	 [11:0] 	pwm_ramp = 0; 

 always @(posedge clk)  begin
	if ( pwm_count_en == 1 ) pwm_ramp <= pwm_ramp + 1; 
   pwm_out 	<= ( pwm_in > pwm_ramp );
 end

endmodule


