`timescale 1ns / 1ps

	/***********************************************************\
	* 				      	VONAL RAJZOLAS 	                     *                                           
	*************************************************************                                           
	* 													                     *                                           
	* 	A kezdopont 	x,  y  regiszterbe                        *                                           
	* 	A vegpont   	x1, y1 regiszterbe 	       		         *                                           
	* 	A vonal szine	color	 regiszter						         *                                           
	* 													                     *                                           
	* 	A vonal rajzolas utan x = x1, y = y1                     *                                           
	* 	Igy olyan vonal  rajzolasanal,  amely az  elozo vegpont- *
	*  jabol indul ki, csak az ujabb vegpont koordinatakat kell *
	*  megadni.  							                           *                                           
	* 													                     *                                           
	* 	3 orajel  kell az  inicializalashoz,  amikor azoknak  a  *
	*  regisztereknek  szamolja ki a  kezdoerteket,  amelyeket  *                                        
	*  a szamitasoknal  konstans ertekkent kasznalok,  illetve  *                                           
	* 	a ciklusvaltozo kezdoerteket allitja be.                 *                                           
	* 	3 orajel kell minden  keppont  kiszamitasahoz, de ennel  *                                           
	* 	lassabban mukodik,  mert varnia  kell  a  memoriakezelo  *                                           
	* 	pont rajzolasra  mert ha a  kepterito  logikanag kell a  *                                           
	* 	ram,  akkor  az  elsobbseget  elvez  es  csak utana tud  *                                           
	* 	pontot rajzolni.   						                     *                                           
	* 	De  igy  is gyorsabban  rajzolja a pontokat, mint ahogy  *                                           
	* 	a display terites mukodik.				                     *                                           
	* 													                     *                                           
	\***********************************************************/

//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    13:00:42 03/20/2011 
// Design Name: 
// Module Name:    line_draw 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module line_draw(	clk,
						start,
						start_x,
						start_y,
						end_x,
						end_y,
						x,
						y,
						plot_start,
						plot_ready,
						ready
					);
 					
input					clk;
input					start;
input		[8:0]		start_x;
input		[7:0]		start_y;
input		[8:0]		end_x;
input		[7:0]		end_y;
output	[8:0]		x;
reg		[8:0]		x;							// Kezdopont koordinata
output	[7:0]		y;
reg		[7:0]		y;							// Kezdopont koordinata
output			plot_start;
reg				plot_start			= 0;
input				plot_ready;
output			ready;


parameter 		plus  = 1;
parameter		minus = 2;
parameter		zero  = 0;

reg	[8:0]		x1;						// Vegpont koordinata
reg	[7:0]		y1;						// Vegpont koordinata
reg	[8:0]		delta_x	 		= 0;
reg	[8:0]		delta_y	 		= 0;
reg	[1:0]		dir_x		 	   = 0;
reg	[1:0]		dir_y		 		= 0;
reg	[9:0]		p			 		= 0;
reg	[9:0]		q			 		= 0;
reg	[8:0]		max_pixel		= 0;
reg	[9:0]		pixel_ct	 		= 0;
reg	[2:0]		line_cikl 		= 0;
reg				busy				= 0;

wire	pixel_ready;

assign	ready = !start && !busy;
assign	pixel_ready	= plot_ready && !plot_start;

always @ ( posedge clk ) begin 

	// LINE_DRAW funkcio keres
   if( start == 1 && busy == 0 ) begin 
		busy 			<= 1;
		line_cikl	<= 0;
		x				<= start_x;
		y				<= start_y;
		x1				<= end_x;
		y1				<= end_y;
	end

	// PLOT/POINT funkcio ready detektalas

	plot_start 	<= 0;
	
	if ( busy == 1 ) begin	
		if ( line_cikl == 0 ) begin
			pixel_ct  <= 1;	// A pont szamlalo kezdoertek
			line_cikl <= line_cikl + 1; // Ciklus inditas

			// Az x es az y lepesek szamanak abszolut erteke 
			if ( x1 >= x ) delta_x <= x1 - x; else delta_x <= x - x1;
			if ( y1 >= y ) delta_y <= y1 - y; else delta_y <= y - y1;
			
			// Az x lepesek iranyanak meghatarozasa
			if ( x1 == x )  begin 
				dir_x <= zero; 
			end
			else begin
				if ( x1 > x ) dir_x <= plus; else dir_x <= minus;
			end
			
			// Az y lepesek iranyanak meghatarozasa
			if ( y1 == y ) begin
				dir_y <= zero;
			end
			else begin
				if ( y1 > y ) dir_y <= plus; else dir_y <= minus;
			end
		end
	
		case ( line_cikl )
			1: begin
					// az egyenes pontjainak szama m-be
					if ( delta_y > delta_x ) max_pixel <= delta_y; else max_pixel <= delta_x;
					line_cikl <= line_cikl + 1;
				end
			
			2: begin
					p 			 <= max_pixel / 2;
					q 			 <= max_pixel / 2;
					line_cikl <= line_cikl + 1;
				end
			3: begin
					// pont kirajzolas
					if ( pixel_ready == 1 ) begin
						plot_start 	<= 1;
						p 			  	<= p + delta_x;
						q 			  	<= q + delta_y;
						line_cikl  	<= line_cikl + 1;
					end
				end
			5: begin // Csak akkor megy tovabb ha a pontrajzolas kesz
					if ( pixel_ready == 1 ) begin 
						line_cikl <= line_cikl + 1;
					end
				end
			4:	begin
					// x leptetese
					if ( p >= max_pixel ) begin
						p <= p - max_pixel;
						if ( dir_x == plus )  x <= x + 1;
						if ( dir_x == minus ) x <= x - 1;
					end
					// y leptetese
					if ( q >= max_pixel ) begin
						q <= q - max_pixel;
						if ( dir_y == plus )  y <= y + 1;
						if ( dir_y == minus ) y <= y - 1;
					end
					// ha pixel_ct=m akkor kesz van a vonal, egyebkent
					// folytatja a vonalrajzolast
					if ( pixel_ct > max_pixel ) begin
						line_cikl 	<= 0;
						pixel_ct		<= 1;
						x 				<= x1;
						y				<= y1;
						busy			<= 0;
					end
					else begin
						// vissza a pontrajzolashoz
						pixel_ct	 <= pixel_ct + 1;
						line_cikl <= 3;
					end
				end
		endcase
	end
end

endmodule

