/*
-------------------------------------------------------------------------------
-- Title      : A Parameterisable Clock Divider
-- Project    : 
-------------------------------------------------------------------------------
-- File       : clk_div.v
-- Author     : Rick Ballantyne rickb@Xilinx.com>
-- Company    : Xilinx, Inc.
-- Created    : 2001/03/15
-- Last Update: 2001/03/15
-- Copyright  : (c) Xilinx Inc, 2001
-------------------------------------------------------------------------------
-- Uses       :   defines.v
-------------------------------------------------------------------------------
-- Used by    :   onewire_iface.v
-------------------------------------------------------------------------------
-- Description: 
--              In this top level module, two SRL16 (16-Bit Shift Register
--              Look-Up-Table) are cascaded to generate a clocks of 1MHz from
--              a system clock with much faster frequency range (3~80MHz)
--              
--              By cascading more SRL16s, this clock divider can accept even
--              higher system frequency.
--		        The divide ratio is defined by the CLK_DIV parameter in the 
--		        "defines.v" file
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  	Description
-- 2001/03/15  1.0      Ballantyne	Conversion from VHDL   

-------------------------------------------------------------------------------
*/
`include "defines.v"

module CLK_DIVIDER (reset,clk_in,clk_out);
input reset;
input clk_in;
output clk_out;

wire clk_out;
reg clk_gen;

wire d;
wire q1;
wire q2;

wire [3:0] sr_addr;
assign sr_addr = `CLK_DIV; // pickup the clock divide number from the defines.v file

assign d =!q2;

assign clk_out = clk_gen;

 
always @ (posedge clk_in or posedge reset) begin
	if (reset) begin
		clk_gen <= 1'b1;
		end
	else begin
		clk_gen <=q2;
		end
	end


SRL16 srl1 (.Q(q1), .A0(sr_addr[0]), .A1(sr_addr[1]), .A2(sr_addr[2]), .A3(sr_addr[3]), .CLK(clk_in), .D(d) )/* synthesis xc_props="INIT =0000"*/;
SRL16 srl2 (.Q(q2), .A0(sr_addr[0]), .A1(sr_addr[1]), .A2(sr_addr[2]), .A3(sr_addr[3]), .CLK(clk_in), .D(q1))/* synthesis xc_props="INIT =0000"*/;       
endmodule

module SRL16(Q, A0, A1, A2, A3, CLK, D); // synthesis syn_black_box
output Q;
input A0;
input A1;
input A2;
input A3;
input CLK;
input D;
endmodule

