`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    09:17:26 03/27/2011 
// Design Name: 
// Module Name:    Sprite 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
// sprite attributum hasznalata eseten 1332-tol vannak az attributum bitek
// 72*74*2 bit pixel + 72*74*1 bit attr  = 1998 byte
// 18*74 byte        + 666 Byte          = 1998 byte
// Ha nem kell attributum, akkor az egesz blokk ram pixel memorianak 
// hasznalhato.

module Sprite( clk,				//  1 bit input 
					start,			//  1 bit input
					x_pos,			//  9 bit input
					y_pos,			//  8 bit input
					width,			//  7 bit input
					height,			//  8 bit input
					x_ct,				//  7 bit input
					y_ct,				//  8 bit input
					ram_addr,		// 11 bit output
					ram_data,		//  8 bit input
					ram_clk,			//  1 bit output
					sprite_data,	//  8 bit output
					sprite_attr,	//  4 bit output
					sprite_attr_en,//	 1 bit input
					ready				//  1 bit output
    );

input					clk;
input					start;			// 1 clk ideig tarto impulzus
input		[8:0]		x_pos;			// a sprite bal felso x koordinataja
input		[7:0]		y_pos;			// a sprite bal felso y koordinataja
input		[6:0]		width;			// a sprite szelessege byte-okban 0-tol.
input		[7:0]		height;			// a sprite magassaga pixelben 0-tol
input		[6:0]		x_ct;				// az lcd terites x koordinataja ( 4 pixel enkent )
input		[7:0] 	y_ct;				// az lcd terites y koordinataja
output	[10:0]	ram_addr;		// blokk ram cim
reg		[10:0]	ram_addr;
input		[7:0]		ram_data;		// blokk ram-bol olvasott adat
output				ram_clk;			// blokk ram orajel
reg					ram_clk;
output	[7:0]		sprite_data;	// 4 pixel adat ( 2bit/pixel )
reg		[7:0]		sprite_data;	
output	[3:0]		sprite_attr;	// 4 pixel attributum ( 0 atlatszo, 1 nem atlatszo )
reg		[3:0]		sprite_attr;
input					sprite_attr_en;// ha 1, akkor engedi az attributum hasznalatat
output				ready;			// 4 pixel adat kiolvasas kesz.
reg					ready;

// Belso regiszterek
reg					sprite_start	= 0;	// ha 1, akkor indul a sprite terites
reg		[2:0]		sprite_cikl		= 0;	// terites allapot regiszter
reg	  [13:0]		data				= 0; 	// Shift regiszter az pixel adatokhoz
reg	   [6:0]		attr				= 0;	// Shift regiszter az attributumokhoz
reg		[6:0]		width_ct			= 0;	// a sprite terites oszlop szamlaloja
reg	  [10:0]		addr				= 0;	// relativ ram cim 

wire					x_ok;						// 1 ha a terites x erteke a sprite-on belul van
wire					y_ok;						// 1 ha a terites y erteke a sprite-on belul van


	// Megvizsgaljuk, hogy az LCD terites a sprite-on belul van-e es ennek megfeleloen
	// jelzi az x es az y tartomanyon beluli allapotot.
	assign	x_ok = ( x_pos[8:2] <= x_ct && { 2'b0, x_pos[8:2] } + width  >= x_ct ) ? 1 : 0;
	assign 	y_ok = ( y_pos      <= y_ct && { 2'b0, y_pos }      + height >= y_ct ) ? 1 : 0;

	always @ ( posedge clk ) begin
	
		// Sprite terites inditas
		if( start == 1 ) begin 
			sprite_cikl	 <= 0;
			sprite_start <= 1;
			ready <= 0;
			// A terites kezdeten minden regiszterbe beirja a kezdo erteket
			if ( x_ct == x_pos[8:2] && y_ct == y_pos ) begin
				data		 <= 0;
				attr		 <= 0;
				addr  	 <= 0;
				width_ct	 <= 0;
				sprite_cikl	 <= 1;
			end
		end

		// Blokk ram orajel csak 1 clk ideig kell es itt van mindig torolve 
		ram_clk <= 0;

		if ( sprite_start == 1 ) begin
			case ( sprite_cikl )
				0:	begin
						// A sprite oszlop szamlalojanak es a ram cimenek a novelese
						if ( width_ct < width  ) begin 
							width_ct <= width_ct + 1;
							addr 		<= addr + 1;
						end
						else begin
							if ( x_pos[8:2] == x_ct ) begin
								width_ct <= 0;
								addr 		<= addr + 1;
							end
						end
						sprite_cikl <= sprite_cikl + 1;
					end
				1: begin
						ram_addr = addr;
						// Attol fuggoen hogy az LCD terites hol tart, engedi vagy
						// tiltja a ram-bol torteno olvasast
						if ( x_ok == 1 && y_ok == 1 ) begin
							//ram_clk		<= 1;
							sprite_cikl <= sprite_cikl + 1;
						end
						else begin
							if ( x_ct != 0 ) begin
								//sprite_cikl <= 5;
								sprite_cikl <= 6;
							end
							else begin
								data 			 <= 0;
								attr 			 <= 0;
								//sprite_cikl	 <= 5;
								sprite_cikl	 <= 6;
							end
						end
					end
				2: begin
						ram_clk		<= 1;
						sprite_cikl <= sprite_cikl + 1;
					end
				3:	begin	
						// Sprite data kiolvasas
						data[7:0] 	<= ram_data;
						// Sprite attr cimzes
						ram_addr		=	1332 + addr[10:1];
						sprite_cikl <= sprite_cikl + 1;
					end
				4:	begin
						ram_clk		<= 1;
						sprite_cikl <= sprite_cikl + 1;
					end
				5:	begin
						// 4 pixel attributum adat kilvasas olvasas
						if ( sprite_attr_en == 1 ) begin 
							if ( addr[0] == 1 ) attr[3:0] <= ram_data[3:0]; else attr[3:0] <= ram_data[7:4];
						end
						else begin
							attr[3:0] <= 0;
						end
						sprite_cikl <= sprite_cikl + 1;
					end
				6:	begin
						// Mivel az x_ct 4 pixelenkent lep ezert az x_pos utolso 2 bitje
						// hatarozza meg, hogy a ram adatot honnal olvassa ki a shift regiszterbol
						case ( x_pos[1:0] )
							0: begin 
									sprite_data <= data[7:0];
									sprite_attr	<=	attr[3:0];
								end
							1: begin 
									sprite_data <= data[9:2];
									sprite_attr	<=	attr[4:1];
								end
							2: begin 
									sprite_data <= data[11:4];
									sprite_attr	<=	attr[5:2];
								end
							3: begin 
									sprite_data <= data[13:6];
									sprite_attr	<=	attr[6:3];
								end
						endcase 
						// Shift regiszterek leptetese
						data 			 <= data << 8;
						attr 			 <= attr << 4;
						sprite_start <= 0;
						ready			 <= 1;
						sprite_cikl	 <= sprite_cikl + 1;
					end
			endcase
		end
	end
endmodule
