`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    11:19:34 03/20/2011 
// Design Name: 
// Module Name:    lcd_sync 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module lcd_sync(
					clk,
					LCD_CP,
					LCD_LOAD,
					LCD_FRAME,
					frame_start,
					lcd_cp_elott,
					char_blink_bit,
					char_blink_rate,
					char_blink_preset,
					graph_blink_rate,
					graph_blink_preset,
					graph_blink_bit,
					lcd_x_ct,
					lcd_y_ct,
					lcd_x_ct_max,
					lcd_y_ct_max,
					refress_rate
					
    );
input				clk;
output			LCD_CP;
reg				LCD_CP 					= 0;	// LCD clock
output			LCD_LOAD;
reg				LCD_LOAD					= 0;	// LCD H sync
output			LCD_FRAME;
reg				LCD_FRAME				= 0;	// LCD V sync
output 			lcd_cp_elott;
reg	 			lcd_cp_elott			= 0;	// LCD_CP jel felfutasa elott 1 clk idore 1-be irodik
output			frame_start;
reg				frame_start				= 0;	// LCD terites kezdete
output	[6:0]	lcd_x_ct;
reg		[6:0]	lcd_x_ct					= 0; 	// LCD oszlop szamlalo 0..79  ( 80 x 4 bit 320 oszlop )
output	[7:0]	lcd_y_ct;
reg		[7:0]	lcd_y_ct					= 0;	// LCD sor szamlalo    0..239 ( 240 sor )
output			lcd_x_ct_max;
reg				lcd_x_ct_max			= 0;	// Ha az lcd_x_ct =  79 akkor 1 egyebkent 0.
output			lcd_y_ct_max;
reg				lcd_y_ct_max			= 0;	// Ha az lcd_x_ct = 239 akkor 1 egyebkent 0.
output			char_blink_bit;
reg				char_blink_bit			= 0;	// Karekteres villogasi utem
input		[3:0]	char_blink_rate;				// Karakter villogasi sebesseg
input				char_blink_preset;
output			graph_blink_bit;				
reg				graph_blink_bit		= 0;	// Grafikus villogasi utem
input		[3:0]	graph_blink_rate;				// Grafikus villogasi sebesseg
input 			graph_blink_preset;

input    [4:0] refress_rate;					// LCD_CP frekvencia

// Belso regiszterek
reg		[7:0]	char_blink_ct			= 0; 	// Karakter villogas szamlalo
reg		[7:0]	graph_blink_ct			= 0; 	// Karakter villogas szamlalo
reg		[6:0]	cp_clk_div				= 0; 	// Elooszto a shift clock jelhez


always @ ( posedge clk ) begin
	/********************************************************************\
	* 	LCD szinkron jelek eloallitasa   (22 50MHz-nel )       			   *	
	* 	LCD_CP Shift clock	2.05MHz (66MHz/32) kb. 72Hz-es frissiteshez  *
	\********************************************************************/
	lcd_cp_elott <= 0;
	if ( LCD_CP == 0 && cp_clk_div == ( 8 + refress_rate ) ) lcd_cp_elott <= 1;
	frame_start <= 0;
	cp_clk_div 	<= cp_clk_div + 1;
	if ( cp_clk_div == ( 9 + refress_rate ) ) begin
		LCD_CP     <= LCD_CP + 1;
		cp_clk_div <= 0;
		if ( LCD_CP == 0 ) begin 
			/****************************************\
			*	LCD sor es oszlop szamlalok           *
			*	lcd_x_ct 0...79 ,   lcd_y_ct 0...239  *
			\****************************************/
			if ( lcd_x_ct == 79 ) begin
				lcd_x_ct <= 0;
				if ( lcd_y_ct == 239 ) begin 
					lcd_y_ct 	<= 0;
					frame_start <= 1;
					
					if ( char_blink_preset == 1 ) begin
						// Uj ertek beirasakor nullazza a szamlalot
						char_blink_ct <= 0;
					end 
					else begin
						if ( char_blink_ct >= { char_blink_rate, 4'b1111 } ) begin
							// Karakteres villogasi sebesseg szamlalo novelese
							char_blink_ct  <= 0;
							char_blink_bit <= ~char_blink_bit;
						end
						else begin
							char_blink_ct <= char_blink_ct + 1; 
						end
					end

					if ( graph_blink_preset == 1 ) begin
						// Uj ertek beirasakor nullazza a szamlalot
						char_blink_ct <= 0;
					end 
					else begin
						if ( graph_blink_ct >= { graph_blink_rate, 4'b1111 } ) begin
						// Grafikus villogasi sebesseg szamlalo novelese
							graph_blink_ct  <= 0;
							graph_blink_bit <= ~graph_blink_bit;
						end
						else begin
							graph_blink_ct <= graph_blink_ct + 1; 
						end
					end
					
				end
				else begin 
					lcd_y_ct <= lcd_y_ct + 1; 
				end
			end
			else begin
				lcd_x_ct <= lcd_x_ct + 1;
			end 
			//*****************************************
		end 
		// Ha az lcd_x_ct elerte a maximalis erteket,
		// akkor az lcd_x_ct_max -ot 1-be irja kulonben 0
		if ( lcd_x_ct == 79  ) lcd_x_ct_max <= 1; else lcd_x_ct_max <= 0;
		if ( lcd_y_ct == 239 ) lcd_y_ct_max <= 1; else lcd_y_ct_max <= 0;
	end
	

	/****************\
	* 	 LCD_LOAD     * 	
	\****************/
	if ( cp_clk_div == 5 && LCD_LOAD == 0 && lcd_x_ct_max == 1 && LCD_CP == 0 ) LCD_LOAD = 1;
	if ( cp_clk_div == 7 && LCD_LOAD == 1 && lcd_x_ct_max == 1 && LCD_CP == 0 ) LCD_LOAD = 0;

	/****************\
	*	 LCD_FRAME    *
	\****************/
	if ( lcd_y_ct == 0 && lcd_x_ct_max == 1 ) LCD_FRAME = 1;
	if ( lcd_y_ct == 1 && lcd_x_ct_max == 1 ) LCD_FRAME = 0;
	
end


endmodule
