
`timescale 1ns / 1ps
	/***********************************************************\
	* 				      	Blokk masolas  	                     *                                           
	*************************************************************                                           
	* 													                     *                                           
	* Forras koordinata x, y regiszterekbe      		            *                                           
	* Cel koordinata x1, y1 regiszterekbe   		  		         *                                           
	* block_mode 0=copy, 1=move, 2=fill						         *                                           
	* Ha block_write_mode = 0 akkor a keppontok kozul a nagyobb *              
	* fenyereju  lessz  kiirva,  ha pedig  1 akkor felulirja  a *                                           
	* forras pont fenyerejevel.                                 *
	* 													                     *                                           
	\***********************************************************/


//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    14:27:16 03/21/2011 
// Design Name: 
// Module Name:    pixel_block 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module pixel_block(	clk,
							start,
							s_x,
							s_y,
							t_x,
							t_y,
							b_w,
							b_h,
							source_layer,
							target_layer,
							source_layer_bp,
							target_layer_bp,
							block_mode,
							block_fill_bright,
							point_pixel_bright,
							plot_point_ready,
							
							plot_start,
							point_start,
							x,
							y,
							layer_select,
							pixel_bp,
							plot_pixel_bright,
							ready
						);

input				clk;
input				start;
input	[8:0]		s_x;
input	[7:0]		s_y;
input	[8:0]		t_x;
input	[7:0]		t_y;
input	[2:0]		source_layer; 			// forras layer kivalasztasa
input	[2:0]		target_layer; 			// cel layer kivalasztasa
input				source_layer_bp;
input				target_layer_bp;
input	[8:0]		b_w;
input	[7:0]		b_h;
input	[4:0]		block_mode; 			
												
input	[1:0]		block_fill_bright;
input				plot_point_ready;
input	 [1:0]	point_pixel_bright;

output			plot_start;
reg				plot_start			= 0;
output			point_start;
reg				point_start			= 0;
output [8:0]	x;
reg	 [8:0]	x						= 0;
output [7:0]	y;
reg	 [7:0]	y						= 0;
output [2:0]	layer_select;
reg	 [2:0]	layer_select		= 0;
output			pixel_bp;
reg				pixel_bp				= 0;
output [1:0]	plot_pixel_bright;
reg	 [1:0]	plot_pixel_bright	= 0;
output			ready;


reg				x_step			   = 0;
reg				y_step			   = 0;
reg	[8:0]		end_x_ct				= 0;
reg	[7:0]		end_y_ct				= 0;
reg	[8:0]		x1						= 0;
reg	[7:0]		y1						= 0;
reg	[3:0]		block_cikl			= 0;
reg	[8:0]		block_x_ct			= 0;
reg	[7:0]		block_y_ct			= 0;
reg				carry_x				= 0;
reg				carry_y				= 0;
reg	[8:0]		source_x				= 0;
reg	[7:0]		source_y				= 0;
reg				busy					= 0;
reg	[1:0]		bright				= 0;
wire				pixel_ready;

reg	[8:0]		block_width		   = 0;
reg	[7:0]		block_height	   = 0;

parameter incr  = 0;
parameter decr  = 1;
parameter x_max = 319;
parameter y_max = 239;


assign	ready = !start && !busy;
assign	pixel_ready	= !plot_start && !point_start && plot_point_ready;

always @ ( posedge clk ) begin 

	// BLOCK funkcio keres
	if( start == 1 && busy == 0 ) begin 
		busy 			 <= 1;
		block_cikl	 <= 0;
		source_x		 <= s_x;
		source_y		 <= s_y;
		x1				 <= t_x;
		y1				 <= t_y;
		block_width  <= b_w - 1;
		block_height <= b_h - 1;
	end
	
	plot_start 	<= 0;
	point_start <= 0;

	
	if ( busy == 1 ) begin
		case ( block_cikl )
			0: begin
					if ( source_x > x1 ) begin 
						block_x_ct <= 0;
						end_x_ct	  <= block_width;
						x_step 	  <= incr;
					end
					else begin 
						block_x_ct <= block_width;
						end_x_ct	  <= 0;
						x_step 	  <= decr;
					end
					
					if ( source_y > y1 ) begin
						block_y_ct <= 0;
						end_y_ct	  <= block_height;
						y_step     <= incr;
					end
					else begin
						block_y_ct <= block_height;
						end_y_ct	  <= 0;
						y_step     <= decr;
					end
					carry_x 		<= 0;
					carry_y 		<= 0;
					block_cikl 	<= block_cikl + 1;
				end
			1:	begin	
					// Forras pont koordinatainak beallitasa
					// carry - be 1 lessz, ha az x vagy az y regiszter
					// szelessegenel nagyobb a kiszamitott koordinata
					{ carry_x, x } <= source_x + block_x_ct;
					{ carry_y, y } <= source_y + block_y_ct;
					block_cikl 	<= block_cikl + 1; 
				end
			2: begin 
					// POINT start
					layer_select <= source_layer;
					pixel_bp		 <= source_layer_bp;
					// Csak ervenyes koordinatak eseten inditja a POINT funkciot
					if ( carry_x == 0 && carry_y == 0 && x <= x_max && y <= y_max ) begin 
						// Pont adatainak kiolvasasa
						point_start <= 1;
					end
					block_cikl 	<= block_cikl + 1; 
				end
			3: begin
					if ( pixel_ready == 1 ) begin
						if ( carry_x == 0 && carry_y == 0 && x <= x_max && y <= y_max ) begin 
							// Ervenyes koordinatak eseten a bright-be a forras pixel fenyereje
							bright <= point_pixel_bright;
						end
						else begin
							// Hibas koordinatak eseten a kiolvasott pont fenyereje = block_fill_bright
							bright <= block_fill_bright;
						end
						carry_x 		<= 0;
						carry_y 		<= 0;
						if ( block_mode[2] == 0 ) block_cikl <= block_cikl + 1; else block_cikl	<= block_cikl + 2;
					end
				end
			4:	begin // COPY or MOVE
					// Cel pixel koordinatak kiszamitasa
					// carry - be 1 lessz, ha az x vagy az y regiszter
					// szelessegenel nagyobb a kiszamitott koordinata
					// MIRROR
					case ( block_mode[4:3] )
						0: begin // NORMAL
								{ carry_x, x } <= x1 + block_x_ct;
								{ carry_y, y } <= y1 + block_y_ct;
							end
						1: begin // MIRROR X
								{ carry_x, x } <= x1 - block_x_ct + block_width;
								{ carry_y, y } <= y1 + block_y_ct;
							end
						2: begin // MIRROR Y
								{ carry_x, x } <= x1 + block_x_ct;
								{ carry_y, y } <= y1 - block_y_ct + block_height;
							end
						3: begin // MIRROR XY
								{ carry_x, x } <= x1 - block_x_ct + block_width;
								{ carry_y, y } <= y1 - block_y_ct + block_height;
							end
					endcase
					// Celkoordinata pixel kiolvasasa a logikai muveletekhez
					point_start <= 1;
					block_cikl <= block_cikl + 2;
				end
			5: begin	// ROTATE
					case ( block_mode[4:3] )
						0: begin // 0 degree rotation
								{ carry_x, x } <= x1 + block_x_ct;
								{ carry_y, y } <= y1 + block_y_ct;
							end
						1: begin // 90 degree rotation
								{ carry_x, x } <= x1 - block_y_ct + block_height;
								{ carry_y, y } <= y1 + block_x_ct;
							end
						2: begin // 180 degree rotation
								{ carry_x, x } <= x1 - block_x_ct + block_width;
								{ carry_y, y } <= y1 - block_y_ct + block_height;
							end
						3: begin // 270 degree rotation
								{ carry_x, x } <= x1 + block_y_ct; 
								{ carry_y, y } <= y1 - block_x_ct + block_width;
							end
					endcase
					// Celkoordinata pixel kiolvasasa a logikai muveletekhez
					point_start <= 1;
					block_cikl <= block_cikl + 1;
				end
			6: begin // A cel pixel kiirasa
					if ( pixel_ready == 1 ) begin
						if ( carry_x == 0 && carry_y == 0 && x <= x_max && y <= y_max ) begin 
							if ( block_mode[1] == 0 ) begin 
								plot_pixel_bright <= bright; // NORMAL
							end
							else begin
								plot_pixel_bright <= block_fill_bright; // FILL
							end
							layer_select <= target_layer;
							pixel_bp		 <= target_layer_bp;
							plot_start 	 <= 1;
						end
						else begin
							// Hibas koordinata eseten nem csinal semmit
						end
						// Pont adatainak kiirasa
						carry_x 		<= 0;
						carry_y 		<= 0;
						block_cikl 	<= block_cikl + 1;
					end
				end
			7:	begin	
					if ( pixel_ready == 1 ) begin 
						// Forras pont koordinatainak beallitasa
						// carry - be 1 lessz, ha az x vagy az y regiszter
						// szelessegenel nagyobb a kiszamitott koordinata
						{ carry_x, x } <= source_x + block_x_ct;
						{ carry_y, y } <= source_y + block_y_ct;
						block_cikl 		<= block_cikl + 1;
					end
				end				
	      8: begin	
					if ( carry_x == 0 && carry_y == 0 && x <= x_max && y <= y_max ) begin 
						if ( block_mode[0] == 1 ) begin 
							// MOVE modban visszairja a pixelt block_fill_bright ertekkel
							layer_select 		<= source_layer;
							pixel_bp		 		<= source_layer_bp;
							plot_pixel_bright <= block_fill_bright;
							plot_start		   <= 1;
						end
					end
					carry_x 		<= 0;
					carry_y 		<= 0;
					block_cikl 	<= block_cikl + 1;
				end
		   9: begin
					if ( pixel_ready == 1 ) begin 
						if ( block_x_ct == end_x_ct && block_y_ct == end_y_ct ) begin
							busy 			<= 0;
							block_cikl  <= 0;
						end
						else begin
							if ( block_x_ct == end_x_ct ) begin
								if ( x_step == incr ) block_x_ct <= 0; else block_x_ct <= block_width;
								if ( y_step == incr ) block_y_ct <= block_y_ct + 1; else block_y_ct <= block_y_ct - 1;
							end
							else begin
								if ( x_step == incr ) block_x_ct <= block_x_ct + 1; else block_x_ct <= block_x_ct - 1;
							end
							block_cikl <= 1;
						end
					end
				end
		endcase
		
	end

end

endmodule
